//
// XMM Science Analysis System
//
// Giuseppe Vacanti (European Space Agency) 
// May 22, 2000
//
#ifndef PROC_H
#define PROC_H

#include <string>
#include <Task.h>
#include <Odf.h>
#include <iostream>
#include<stdexcept>

class ProcException : public std::exception {
public:
  ProcException(const char * msg = "");
  virtual const char * what() const throw();
private:
  const char * msg_;
};


class Proc {

 public:

  // A wrapper around existing facilities
  class Task {
  public:
    Task(std::string name, bool resetparams = true);
    template<class T> void setParameter(const std::string& name, const T& value);
    template<class T> void getParameter(const std::string& name, T& value) const;
    void run(bool invoke=false);

  private:
    Info::Task * task_;
    ParameterInfo & params_;
  };


  // Auxiliary set
  struct AuxSet {
    const OdfSet* set;
  };

  // Science exposure: event and auxiliary sets
  struct ScienceExposure {
    Xmm::DataMode mode;
    AuxSet auxset;
    const Exposure* exposure;
    bool hasauxset;
    int index;
  };


  Proc(const std::string& directory );

  // Run a task with exceptions on
  bool protectedrun(Info::Task* t, const bool system = true);

  // Remove the temporary files
  void remove_temporaries();
  
  // Remove intermediate GTIs
  void remove_gtis();
  
  // Remove pileup temp files
  void remove_pileupTempFiles();

  // Remove the rest
  void remove_last();

  // File naming convention. The OAL state must be set correctly.
  //
  // These are hierarchically structured. With or without a suffix.
  std::string observationbasename(const bool with_suffix = false);
  std::string instrumentbasename(const bool with_suffix = false);
  std::string exposurebasename(const bool with_suffix = false);
  std::string ccdbasename(const bool with_suffix = false);
  std::string nodebasename(const bool with_suffix = false);

  // These are complete names.
  //
  // observation
  std::string hkgtiname();
  std::string atthkgenname();
  std::string observationgtiname();

  // instrument
  std::string instrumentgtiname();

  // exposure
  std::string auxgtiname();

  // ccd
  std::string framesgtiname();
  std::string ccdgtiname();
  std::string ccdgtitablename(const bool qualified = false);
  std::string eventsname(const bool filtered = false);

  std::string pileupname(const bool filtered=false);


  std::string imagingeventsname(const bool filtered = false);
  std::string timingeventsname(const bool filtered = false);
  std::string bursteventsname(const bool filtered = false);

  // These are temporary names
  std::string epauxcombname();
  std::string timeappendname();
  std::string framesname();
  //  std::string epframesname();
  std::string badpixfindname();
  //  std::string emframesname();
  //  std::string emframeseventsname();
  std::string frameseventsname();
  std::string badpixfindeventsname();
  std::string gattispectrumname();
  //AI: Temporary names for the new calls to emeventsproj
  std::string emeventsprojeventsname();
  
  //AI: Needed for the call to epreject
  std::string eprejecteventsname();

  //AI: Needed for epxrlcorr call
  std::string epxrlcorreventsname();

  //AI: Needed for evlistcomb
  std::string tempImagingname();
  std::string tempTimingname();

  //AI: Needed for evselect while merging the files.
  std::string tempImagingEvlistname();
  std::string tempTimingEvlistname();

  //AI: Name for the Epic flare background timeserie
  std::string tempFlareBkgTimeseriename();
  std::string tempFlareBkgGTIname();

  void print(const ScienceExposure& e, ostream& os);
  
 private:

  const std::string dir_;
  const std::string suffix_;

  // Temporaries relevant for one exposure
  std::vector<std::string> temporaries_;
  // GTIs relevant for one exposure
  std::vector<std::string> gtis_;
  //Pileup temporary files
  std::vector<std::string> pileup_;
  // Other temporary/GTI files that have to be kept for the whole
  // observation. For example the HKGTI.
  std::vector<std::string> cleanlast_;

  
};

#endif
